turtles-own [
  cash             ;; amount of cash of the agent
  WTP              ;; willingness to pay
  WTA              ;; willingness to accept the proposal 
  status           ;; defines the role of the agents [0 = not-playing, 1 = proposer, 2 = receiver]
  played?          ;; this is a flag used to mark agents that already played a game at a certain tick
  behav            ;; behaviour of the agent (selfish = 0; altruist = 1)
]

;; This procedure is used to toss a coin in order to decide the roles of each agents that is playing.
to-report toss-coin? [coin]   
  ifelse (random-float 1) >= coin
  [ report true ]
  [ report false ]
end

;; Verify if it is possible to stop the simulation at a certain tick.
;; Here is possibile to adjust the value to run indefinitely the simulation.
to-report stop?
  ifelse ticks = 750 [report true] [report false]
end


;; SETUP PROCEDURE
;; Prepares the world asking to N patches to create 1 agent and initialiaze its variables.
;; In the interface tab must be inserted a button calling this procedure.
to setup
  clear-all
  ;; Here, "N" indicates the number of the agents that the program must generates.
  ;; The value is adjustable by a slider in the interface tab ranging from 2 to 1.000.
  ask n-of N patches
  [
    ;; Asked patches create 1 turtle. In this way, at time 0 there is no chance to have
    ;; more than 1 agent on the same patch.
    sprout 1
    [
      set shape "person"
      ;; set a random starting point in world.
      setxy random-xcor random-ycor
      set cash 0
      ;; set the WTA as fixed in the interface tab by the slider "minimum-money-accepted".
      set WTA minimum-money-accepted
    ]
  ]
  initBehav
  resetGames
  reset-ticks
end

;; This procedure is used to initialize for the first time agents' WTP, color and behaviour.
to initBehav
  ;; Initialiaze selfish agents.
  ask turtles
  [ set WTP (35 + random 11)
    set color gray
    set behav 0]
  ;; Initialiaze altruist agents.
  ;; Here, "%altruist" indicates the percentage of agents that show an altruistic behaviour.
  ;; This value is alterable by the interface tab trough a slider.
  ask n-of (N * %altruist) turtles
  [ set WTP (45 + random 11)
    set color white
    set behav 1]
end

;; The procedure reset several conditions the agents in order to prepare them to play
;; once again the ultimatum game.
to resetGames
  ask patches [set pcolor black]
  ask turtles
  [ set played? false
    set status 0
    ifelse (behav = 0)
    [ set color grey ]
    [ set color white ]
  ]
end


;; GO PROCEDURE
;; This procedure is the engine of the simulation.
;; First, it asks all agents to move; then, it searches a player which has not played at that tick and couple it with one other free player.
;; In the interface tab must be inserted a button calling this procedure (check "Forever" to run the procedure continuously, it will stop
;; when ticks are equeal to 750).
to go
  resetGames
  ask turtles
  [
    rt random 360
    fd random-float 5
  ]
  ask turtles 
  [
    if (played? = false)
    [
      let otherPlayers nobody
      set otherPlayers other turtles-here with [played? = false]
      if any? otherPlayers 
      [
        let otherPlayer nobody
        set otherPlayer one-of otherPlayers
        ask otherPlayer
        [
          set played? true
        ] 
        set played? true
        set pcolor 67     ;; paint the patch with, at least, two agents on it
        toss-n-play otherPlayer
      ]
    ]
  ]
  updatePlots
  tick
  if stop? [ stop ]
end

;; The tossed coin set the roles of each player and the colors (red and green) visualize them on the world.
;; With a probability of .50 the called agent take the role of proposer (and so, it is endowed with 100€)
;; and viceversa if p < .50. The other agent playing the game will take the opposite role.
to toss-n-play [ player2 ]
  ifelse toss-coin? 0.5
  ;; set the first agent as proposer and the second as receiver
  [ set status 1
    set cash cash + 100
    set color red
    ask player2 
    [ set status 2
      set color green ] ]
  ;; in opposition with the previous code, this set the first agent as recevied and the second as proposer
  [ set status 2
    set color green
    ask player2
    [ set status 1
      set cash cash + 100
      set color red ] ]
  
;; The agents play Ultimatum Game with a single bargaining and not knowing the WTA of other agent.
;; If status of the agent (the one called by the "ask" inside the Go procedure) is 1, he will play as proposer (so we consider its WTP).
  if status = 1
  [
    ifelse WTP > [WTA] of player2
    [
      ;; both agents win the game
      set cash cash - WTP 
      let bonus WTP
      ask player2 [set cash cash + bonus]
    ]
    [
      ;; called agent is the proposer and loses the game
      set cash cash - 100 
    ]
  ]
;; If status of the agent is 2, he will play as receiver (so we consider its WTA)
  if status = 2
  [
    ifelse WTA < [WTP] of player2
    [
      ;; both agents win the game
      set cash cash + [WTP] of player2
      ask player2 [set cash cash - WTP]
    ]
    [
      ;; player2 is the proposer and loses the game
      ask player2 [set cash cash - 100]
    ]
  ]
end

;; This procedure updates the plot on the interface tab that measures at each tick the mean cash earned by agents accordingly to their own group.
to updatePlots
 if (count turtles with [behav = 0]) > 0
 [ set-current-plot-pen "$ selfish agents" plot (mean [cash] of turtles with [behav = 0])]
 if (count turtles with [behav = 1]) > 0
 [ set-current-plot-pen "$ Altruist agents" plot (mean [cash] of turtles with [behav = 1])]
end
@#$#@#$#@
GRAPHICS-WINDOW
206
17
645
477
16
16
13.0
1
10
1
1
1
0
0
0
1
-16
16
-16
16
0
0
1
ticks
30.0

BUTTON
16
154
80
187
Setup
setup
NIL
1
T
OBSERVER
NIL
S
NIL
NIL
1

BUTTON
100
154
186
187
Go (once)
go
NIL
1
T
OBSERVER
NIL
G
NIL
NIL
1

BUTTON
14
211
184
244
Run
go
T
1
T
OBSERVER
NIL
R
NIL
NIL
1

MONITOR
771
37
821
82
N°
count turtles with [behav = 0]
17
1
11

MONITOR
1031
37
1081
82
N°
count turtles with [behav = 1]
17
1
11

SLIDER
17
26
189
59
N
N
10
1000
100
2
1
NIL
HORIZONTAL

MONITOR
678
34
738
79
#Players
count turtles with [color = green or color = red]
17
1
11

MONITOR
832
37
924
82
Avg. group cash
precision (mean [cash] of turtles with [behav = 0]) 2
17
1
11

MONITOR
1091
38
1184
83
Avg. group cash
precision (mean [cash] of turtles with [behav = 1]) 2
17
1
11

PLOT
678
172
1311
433
Mean cash of agents
Tick
Avg. cash
0.0
10.0
0.0
10.0
true
false
"" ""
PENS
"$ Selfish agents" 1.0 0 -7500403 true "" ""
"$ Altruist agents" 1.0 0 -16777216 true "" ""

SLIDER
16
104
189
137
minimum-money-accepted
minimum-money-accepted
15
55
39
1
1
NIL
HORIZONTAL

MONITOR
832
96
925
141
Avg. group WTP
precision (mean [WTP] of turtles with [behav = 0]) 2
17
1
11

MONITOR
1091
96
1184
141
Avg. group WTP
precision (mean [WTP] of turtles with [behav = 1]) 2
17
1
11

TEXTBOX
771
17
921
35
Selfish Players\n
13
5.0
1

TEXTBOX
1033
17
1183
35
Altruist Players
13
0.0
1

TEXTBOX
713
192
863
210
Selfish agents
13
5.0
1

TEXTBOX
713
209
863
227
Altruist agents
13
0.0
1

MONITOR
771
96
823
141
Min WTP
min [WTP] of turtles with [behav = 0]
17
1
11

MONITOR
934
96
991
141
Max WTP
max [WTP] of turtles with [behav = 0]
17
1
11

MONITOR
1031
97
1084
142
Min WTP
min [WTP] of turtles with [behav = 1]
17
1
11

MONITOR
1191
97
1248
142
Max WTP
max [WTP] of turtles with [behav = 1]
17
1
11

TEXTBOX
16
197
166
215
Simulation stops at 750 ticks.
11
0.0
1

SLIDER
16
64
188
97
%altruist
%altruist
0
1
0.35
0.05
1
NIL
HORIZONTAL

MONITOR
12
254
95
299
Avg. Proposal
precision (mean [WTP] of turtles) 2
17
1
11

@#$#@#$#@
## WHAT IS IT?

The simulation replicates N agents playing randomly the Ultimatum Game (UG) among themselves.

## HOW IT WORKS

The setup will arrange N agents in the world. Agents' coordinates are randomly assigned and the initialization of the parameters are derived by the interface tab sliders. Grey agents indicate a selfish behaviours, whereas white ones indicates altruistic proposals.

At each tick the code works as follow:

 * Ask all agents to move randomly in the world.

 * Ask to each agent if they already played a game in the tick. If not, the code asks to the agent if there are other agents on the same patch that haven’t played yet. If true, the first agent choose (randomly) a second player that hasn't played and play the ultimatum game with him for the fixed amount of the 100,00€. That is like to say that the agents are arrived at that patch "simultaneously" and find 100,00€, and now they have to split it in some way: so, they decide to play UG.

  * The roles of the two agents is defined through an hypothetical flip of a coin (that is, it is based on a probability of 50/50). Hence, the coin defines the roles of proposer and receiver for both agents playing (they are marked, respectively, with red and green colour to be easily detectable on the virtual world). The game rules are the following:
    * if the proposal (so, the willingness to pay, WTP) of player-1 is greater than the minimum of acceptance of the receiver (so, the willingness to accept, WTA) of player-2 they both win. The receiver will earn the amount of money of the proposal and the proposer will earn the rest of the money (equal to [100 – proposal]).
    * if the proposal of player-1 is lower than the minimum of acceptance of player-2 they both lose. The receiver will not earn anything and the proposer will lose the money played in the game from its purse.

* At the next tick the agents are rearranged to play another time (so, the status of proposal/receiver is removed and the colour changes to the original setup).

The amount of money that two agents need to split between them is fixed at 100€ and all start with zero money.

## HOW TO USE IT

Interface is composed by two input:

 * “N” indicates the number of agents to set in the world.

 * “minimum-money-accepted” represents the limit that agents have to accept the proposal (namely, WTA). If the proposal is under of this value, an agent will refuse it (read as an *altruistic punishment*) and both agents playing lose the game.

 * "%altruist" indicates the percentage of altruist agents that the program must implement into the current simulation.


## THINGS TO NOTICE

-

## THINGS TO TRY

-

## EXTENDING THE MODEL

-

## NETLOGO FEATURES

-

## RELATED MODELS

-

## CREDITS AND REFERENCES

Vers.1.4. – January 9th 2015 by A. Scalco (andrea.scalco@univr.it)
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.1.0
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="PAAMS-experiment" repetitions="10" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <metric>precision (mean [WTP] of turtles) 2</metric>
    <metric>precision (mean [cash] of turtles with [behav = 1]) 2</metric>
    <metric>precision (mean [WTP] of turtles with [behav = 1]) 2</metric>
    <metric>precision (mean [cash] of turtles with [behav = 0]) 2</metric>
    <metric>precision (mean [WTP] of turtles with [behav = 0]) 2</metric>
    <enumeratedValueSet variable="%altruist">
      <value value="0.5"/>
    </enumeratedValueSet>
    <steppedValueSet variable="minimum-money-accepted" first="30" step="1" last="40"/>
    <enumeratedValueSet variable="N">
      <value value="1000"/>
    </enumeratedValueSet>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
